==> concierge-1.2.0-sources.jar: com/flybits/concierge/fragments/CategoryFragment.kt <==
package com.flybits.concierge.fragments

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v4.app.FragmentActivity
import android.support.v4.widget.SwipeRefreshLayout
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.flybits.commons.library.logging.Logger
import com.flybits.concierge.ConciergeConstants
import com.flybits.concierge.ConciergeFragment
import com.flybits.concierge.FlybitsViewProviderGetter
import com.flybits.concierge.R
import com.flybits.concierge.adapters.FeedAdapter
import com.flybits.concierge.viewmodels.CategoryViewModel
import com.flybits.concierge.viewmodels.CategoryViewModelFactory
import com.flybits.concierge.viewmodels.TwitterViewModel

class CategoryFragment : BasePagerFragment(), CategoryViewModel.FeedErrorDisplayer {

    companion object {
        const val CATEGORY_EXTRA = "category"

        fun newInstance(category: String): CategoryFragment {
            val bundle = Bundle()
            bundle.putString(CATEGORY_EXTRA, category)
            val fragment = CategoryFragment()
            fragment.arguments = bundle
            return fragment
        }
    }

    private var recyclerView: RecyclerView? = null
    private var categoryViewModel: CategoryViewModel? = null
    private var feedAdapter: FeedAdapter? = null
    private var noDataView: View? = null
    private var refreshReceiver: BroadcastReceiver? = null
    private var swipeRefreshLayout: SwipeRefreshLayout? = null
    private var firstStart = true

    override fun titleRes(): Int {
        return R.string.flybits_con_fragment_title_feed
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.flybits_con_fragment_category, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        Logger.d("Category: ${arguments!!.getString("category","")} fragment view created!")

        val linearLayoutManager = LinearLayoutManager(context)
        recyclerView = view.findViewById(R.id.flybits_con_fragment_feed_lytRecycler)
        recyclerView?.layoutManager = linearLayoutManager
        val conciergeFragment = activity?.supportFragmentManager?.fragments?.find { it is ConciergeFragment } as ConciergeFragment
        feedAdapter = FeedAdapter(conciergeFragment, conciergeFragment)
        recyclerView?.adapter = feedAdapter
        recyclerView?.setHasFixedSize(false)
        recyclerView?.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView?, dx: Int, dy: Int) {
                val firstVisibleItem = linearLayoutManager.findFirstCompletelyVisibleItemPosition()
                val lastVisibleItem = linearLayoutManager.findLastCompletelyVisibleItemPosition()
                if (firstVisibleItem != RecyclerView.NO_POSITION && lastVisibleItem != RecyclerView.NO_POSITION){
                    val items = feedAdapter?.currentList?.subList(firstVisibleItem, lastVisibleItem+1)
                    items?.filter { it != null }?.let {
                        categoryViewModel?.onScroll(it)
                    }
                }
                super.onScrolled(recyclerView, dx, dy)
            }
        })

        //Logic for refresh animations after the first data load
        swipeRefreshLayout = view.findViewById(R.id.flybits_con_swipe_ref_layout)
        swipeRefreshLayout?.setOnRefreshListener {
            Logger.d(CategoryFragment::class.java.simpleName + ": onRefresh()")

            categoryViewModel?.refresh{
                //hide animation once view model notifies us that its finished
                swipeRefreshLayout?.isRefreshing = false
            }

            if (context != null) {
                val twitterViewModel = ViewModelProviders
                        .of(context as FragmentActivity)
                        .get(TwitterViewModel::class.java)

                twitterViewModel.clearTweetLists()
            }
        }

        noDataView = view.findViewById(R.id.flybits_con_fragment_feed_lytNoData)

        categoryViewModel = ViewModelProviders.of(this, CategoryViewModelFactory(context!!
                , conciergeFragment as FlybitsViewProviderGetter, arguments!!.getString(CATEGORY_EXTRA)))
                .get(CategoryViewModel::class.java)

        //First load, show refresh animation
        swipeRefreshLayout?.isRefreshing = true
        categoryViewModel?.feedContent
                ?.observe(this, Observer { baseTemplates ->
                    baseTemplates?.let {
                        Logger.d("Category: ${arguments!!.getString("category","")}, Got data! data: $it")
                        feedAdapter?.submitList(it)
                        val baseTemplatesEmpty = it.isEmpty()
                        recyclerView?.visibility = if (baseTemplatesEmpty) View.GONE else View.VISIBLE
                        noDataView?.visibility = if (baseTemplatesEmpty) View.VISIBLE else View.GONE
                        swipeRefreshLayout?.isRefreshing = false
                    }
                })
        categoryViewModel?.setFeedErrorDisplayer(this)

        refreshReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context, intent: Intent) {
                if (feedAdapter != null) {
                    try {
                        feedAdapter?.notifyDataSetChanged()
                    } catch (e: Exception) {
                        Logger.exception("CategoryFragment.RefreshReceiver.onReceiver()", e)
                    }

                }
            }
        }

        //Refresh data to make sure local data is in sync with server
        categoryViewModel?.refresh()
    }

    //Refresh content if activity is brought to start state ONLY after it is called more than once
    //This refreshes the content when the app is brought into the foreground from the background
    override fun onStart() {
        super.onStart()
        if (firstStart){
            firstStart = false
        } else {
            swipeRefreshLayout?.isRefreshing = true
            categoryViewModel?.refresh {
                //hide animation once view model notifies us that its finished
                swipeRefreshLayout?.isRefreshing = false
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        try {
            val activity = activity
            activity?.registerReceiver(refreshReceiver, IntentFilter(ConciergeConstants.BROADCAST_AUDIO_STATE))
        } catch (e: Exception) {
            Logger.exception("CategoryFragment.onCreate()", e)
        }

    }

    override fun onDestroy() {
        super.onDestroy()
        //Receiver may already be unregistered or may have never been registered in that case catch the exception
        try {
            val activity = activity
            activity?.unregisterReceiver(refreshReceiver)
        } catch (e: Exception) {
            Logger.exception("CategoryFragment.onDestroy()", e)
        }

    }

    override fun onError(err: String) {
        Logger.d("CategoryFragment: onError() err: $err")
        val activity = activity
        val v = view
        /*The instanceof check is done to make sure that the user is still within
        * the concierge feature at the time the toast is displayed.*/
        if (activity != null && v != null) {
            Snackbar.make(v, err, Snackbar.LENGTH_LONG).show()
        }
    }
}

==> concierge-1.3.0-sources.jar: com/flybits/concierge/fragments/CategoryFragment.kt <==
package com.flybits.concierge.fragments

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v4.app.FragmentActivity
import android.support.v4.widget.SwipeRefreshLayout
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.flybits.commons.library.logging.Logger
import com.flybits.concierge.ConciergeConstants
import com.flybits.concierge.ConciergeFragment
import com.flybits.concierge.FlybitsViewProviderGetter
import com.flybits.concierge.R
import com.flybits.concierge.adapters.FeedAdapter
import com.flybits.concierge.models.Category
import com.flybits.concierge.viewmodels.CategoryViewModel
import com.flybits.concierge.viewmodels.CategoryViewModelFactory
import com.flybits.concierge.viewmodels.TwitterViewModel

/**
 * This fragment is responsible for displaying content that belongs to a particular
 * [Category]. Instantiate this fragment using the exposed `newInstance(Category)` method.
 */
class CategoryFragment : BasePagerFragment(), CategoryViewModel.FeedErrorDisplayer {

    companion object {
        const val CATEGORY_EXTRA = "category"

        fun newInstance(category: Category): CategoryFragment {
            val bundle = Bundle()
            bundle.putParcelable(CATEGORY_EXTRA, category)
            val fragment = CategoryFragment()
            fragment.arguments = bundle
            return fragment
        }
    }

    private var recyclerView: RecyclerView? = null
    private var categoryViewModel: CategoryViewModel? = null
    private var feedAdapter: FeedAdapter? = null
    private var noDataView: View? = null
    private var refreshReceiver: BroadcastReceiver? = null
    private var swipeRefreshLayout: SwipeRefreshLayout? = null
    private var firstStart = true

    override fun titleRes(): Int {
        return R.string.flybits_con_fragment_title_feed
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.flybits_con_fragment_category, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val category = arguments?.getParcelable<Category>(CATEGORY_EXTRA)
        val context = context

        Logger.d("Category: $category fragment view created!")

        val linearLayoutManager = LinearLayoutManager(context)
        recyclerView = view.findViewById(R.id.flybits_con_fragment_feed_lytRecycler)
        recyclerView?.layoutManager = linearLayoutManager
        val conciergeFragment = activity?.supportFragmentManager?.fragments?.find { it is ConciergeFragment } as ConciergeFragment
        feedAdapter = FeedAdapter(conciergeFragment, conciergeFragment)
        recyclerView?.adapter = feedAdapter
        recyclerView?.setHasFixedSize(false)
        recyclerView?.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView?, dx: Int, dy: Int) {
                val firstVisibleItem = linearLayoutManager.findFirstCompletelyVisibleItemPosition()
                val lastVisibleItem = linearLayoutManager.findLastCompletelyVisibleItemPosition()
                if (firstVisibleItem != RecyclerView.NO_POSITION && lastVisibleItem != RecyclerView.NO_POSITION){
                    val items = feedAdapter?.currentList?.subList(firstVisibleItem, lastVisibleItem+1)
                    items?.filter { it != null }?.let {
                        categoryViewModel?.onScroll(it)
                    }
                }
                super.onScrolled(recyclerView, dx, dy)
            }
        })

        //Logic for refresh animations after the first data load
        swipeRefreshLayout = view.findViewById(R.id.flybits_con_swipe_ref_layout)
        swipeRefreshLayout?.setOnRefreshListener {
            Logger.d(CategoryFragment::class.java.simpleName + ": onRefresh()")

            categoryViewModel?.refresh{
                //hide animation once view model notifies us that its finished
                swipeRefreshLayout?.isRefreshing = false
            }

            if (context != null) {
                val twitterViewModel = ViewModelProviders
                        .of(context as FragmentActivity)
                        .get(TwitterViewModel::class.java)

                twitterViewModel.clearTweetLists()
            }
        }

        noDataView = view.findViewById(R.id.flybits_con_fragment_feed_lytNoData)

        if (category != null && context != null){
            categoryViewModel = ViewModelProviders.of(this, CategoryViewModelFactory(context
                    , conciergeFragment as FlybitsViewProviderGetter, category)).get(CategoryViewModel::class.java)
        }

        //First load, show refresh animation
        swipeRefreshLayout?.isRefreshing = true
        categoryViewModel?.feedContent
                ?.observe(this, Observer { baseTemplates ->
                    baseTemplates?.let {
                        Logger.d("Category: $category, Got data! data: $it")
                        feedAdapter?.submitList(it)
                        val baseTemplatesEmpty = it.isEmpty()
                        recyclerView?.visibility = if (baseTemplatesEmpty) View.GONE else View.VISIBLE
                        noDataView?.visibility = if (baseTemplatesEmpty) View.VISIBLE else View.GONE
                        swipeRefreshLayout?.isRefreshing = false
                    }
                })
        categoryViewModel?.setFeedErrorDisplayer(this)

        refreshReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context, intent: Intent) {
                if (feedAdapter != null) {
                    try {
                        feedAdapter?.notifyDataSetChanged()
                    } catch (e: Exception) {
                        Logger.exception("CategoryFragment.RefreshReceiver.onReceiver()", e)
                    }

                }
            }
        }

        //Refresh data to make sure local data is in sync with server
        categoryViewModel?.refresh()
    }

    //Refresh content if activity is brought to start state ONLY after it is called more than once
    //This refreshes the content when the app is brought into the foreground from the background
    override fun onStart() {
        super.onStart()
        if (firstStart){
            firstStart = false
        } else {
            swipeRefreshLayout?.isRefreshing = true
            categoryViewModel?.refresh {
                //hide animation once view model notifies us that its finished
                swipeRefreshLayout?.isRefreshing = false
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        try {
            val activity = activity
            activity?.registerReceiver(refreshReceiver, IntentFilter(ConciergeConstants.BROADCAST_AUDIO_STATE))
        } catch (e: Exception) {
            Logger.exception("CategoryFragment.onCreate()", e)
        }

    }

    override fun onDestroy() {
        super.onDestroy()
        //Receiver may already be unregistered or may have never been registered in that case catch the exception
        try {
            val activity = activity
            activity?.unregisterReceiver(refreshReceiver)
        } catch (e: Exception) {
            Logger.exception("CategoryFragment.onDestroy()", e)
        }

    }

    override fun onError(err: String) {
        Logger.d("CategoryFragment: onError() err: $err")
        val activity = activity
        val v = view
        /*The instanceof check is done to make sure that the user is still within
        * the concierge feature at the time the toast is displayed.*/
        if (activity != null && v != null) {
            Snackbar.make(v, err, Snackbar.LENGTH_LONG).show()
        }
    }
}
